﻿using System;
using System.Collections.Specialized;
using System.Xml.Linq;
using System.Net;

namespace Anis.Twitter.Api {
    public class Methods {
        #region URL定数
        
        const string URL_blocks_ = "http://twitter.com/blocks/";
        const string URL_direct_messages = "http://twitter.com/direct_messages";
        const string URL_favorites = "http://twitter.com/favorites";
        const string URL_favourings = "http://twitter.com/favourings";
        const string URL_friendships_ = "http://twitter.com/friendships/";
        const string URL_statuses = "http://twitter.com/statuses";
        const string URL_statuses_featured_xml = "http://twitter.com/statuses/featured.xml";
        const string URL_statuses_followers = "http://twitter.com/statuses/followers";
        const string URL_statuses_friends = "http://twitter.com/statuses/friends";
        const string URL_users = "http://twitter.com/users";
        
        #endregion

        #region 初期化関連
        
        /// <summary>
        /// UserIDとUserPassを指定してStatusQueryを初期化する
        /// </summary>
        /// <remarks>ログインはしない。</remarks>
        public Methods(string userID, string password) {
            this.UserID = userID;
            this.UserPass = password;
        }

        /// <summary>
        /// ユーザー認証に使用するメールアドレス、またはスクリーン名
        /// </summary>
        public string UserID { set; get; }

        /// <summary>
        /// ユーザー認証に使用するパスワード
        /// </summary>
        public string UserPass { set; get; }

        #endregion

        #region ブロック関連未test

        //todo:test,戻り値不明
        /// <summary>
        /// 指定ユーザをブロックする
        /// </summary>
        /// <param name="user">ブロックしたいユーザを指定する</param>
        public string AddBlockUser(User user) {
#if DEBUG
            return Utility.HttpGet(URL_blocks_ + "create/" + user.Id + ".xml", new NameValueCollection(), this.UserID, this.UserPass);
#else
            throw new NotImplementedException();
#endif
        }

        //todo:test,戻り値不明
        /// <summary>
        /// 指定ユーザのブロックを解除する
        /// </summary>
        /// <param name="user">ブロック解除したいユーザを指定する</param>
        public string RemoveBlockUser(User user) {
#if DEBUG
            return Utility.HttpGet(URL_blocks_ + "destroy/" + user.Id + ".xml", new NameValueCollection(), this.UserID, this.UserPass);
#else
            throw new NotImplementedException();
#endif
        }
        
        #endregion

        #region ダイレクトメッセージ関連未test

        //todo:test
        /// <summary>
        /// 自分宛てのダイレクトメッセージの一覧を取得する (最大20件)
        /// </summary>
        public DirectMessage[] GetDirectMessages() {
            return GetDirectMessages(null, null, 0);
        }
        //todo:test
        /// <summary>
        /// 自分宛てのダイレクトメッセージの一覧を取得する (最大20件)
        /// </summary>
        /// <param name="since">指定した日時以降に届いたダイレクトメッセージを取得する</param>
        /// <param name="sinceID">指定したIDより大きな値のIDのダイレクトメッセージを最大20件取得する</param>
        /// <param name="page">(1ページを20件とみなしたときの)ページ番号を指定することで、過去の任意のダイレクトメッセージを20件単位で取得する</param>
        public DirectMessage[] GetDirectMessages(Nullable<DateTime> since, DirectMessage sinceID, int page) {
#if DEBUG
            NameValueCollection options = new NameValueCollection();
            if(since != null) {
                options.Add("since", Utility.ConvertDateTime(since.Value));
            }
            if(sinceID != null) {
                options.Add("since_id", sinceID.ID.ToString());
            }
            if(page != 0) {
                options.Add("page", page.ToString());
            }
            string res = Utility.HttpGet(URL_direct_messages + ".xml", options, this.UserID, this.UserPass);

            return DirectMessage.CreateDirectMessages(res);
#else
            throw new NotImplementedException();
#endif
        }

        //todo:test
        /// <summary>
        /// 自分が送信したダイレクトメッセージの一覧を取得する (最大20件)
        /// </summary>
        public DirectMessage[] GetSendDirectMessages() {
            return GetSendDirectMessages(null, null, 0);
        }
        //todo:test
        /// <summary>
        /// 自分が送信したダイレクトメッセージの一覧を取得する (最大20件)
        /// </summary>
        /// <param name="since">指定した日時以降に送信したダイレクトメッセージを取得する</param>
        /// <param name="sinceID">指定したIDより大きな値のIDの送信済みダイレクトメッセージを最大20件取得する</param>
        /// <param name="page">(1ページを20件とみなしたときの)ページ番号を指定することで、過去の任意の送信済みダイレクトメッセージを20件単位で取得する</param>
        public DirectMessage[] GetSendDirectMessages(Nullable<DateTime> since, DirectMessage sinceID, int page) {
#if DEBUG
            NameValueCollection options = new NameValueCollection();
            if(since != null) {
                options.Add("since", Utility.ConvertDateTime(since.Value));
            }
            if(sinceID != null) {
                options.Add("since_id", sinceID.ID.ToString());
            }
            if(page != 0) {
                options.Add("page", page.ToString());
            }
            string res = Utility.HttpGet(URL_direct_messages + "/sent.xml", options, this.UserID, this.UserPass);

            return DirectMessage.CreateDirectMessages(res);
#else
            throw new NotImplementedException();
#endif
        }

        //todo:test,戻り値不明
        /// <summary>
        /// ダイレクトメッセージを送信する。宛先と本文の指定は必須。
        /// </summary>
        /// <param name="user">ユーザID または user=スクリーン名  (必須)</param>
        /// <param name="text">ダイレクトメッセージの本文を指定する。必ず URL エンコードすること。本文は140バイト以内におさめること。</param>
        public string SendDirectMessage(User user, string text) {
#if DEBUG
            NameValueCollection options = new NameValueCollection();
            options.Add("user", user.Id.ToString());
            options.Add("text", text);
            return Utility.HttpPost(URL_direct_messages + "/new.xml", options, this.UserID, this.UserPass);
#else
            throw new NotImplementedException();
#endif
        }

        //todo:test,戻り値不明
        /// <summary>
        /// ダイレクトメッセージを削除する。メッセージの指定は必須。
        /// </summary>
        /// <param name="id">削除したいダイレクトメッセージを指定する</param>
        public string DeleteDirectMessage(DirectMessage id) {
#if DEBUG
            return Utility.HttpGet(URL_direct_messages + "/destroy/" + id.ID + ".xml", new NameValueCollection(), this.UserID, this.UserPass);
#else
            throw new NotImplementedException();
#endif
        }

        #endregion

        #region お気に入り関連

        /// <summary>
        /// 自分のfavorites(お気に入り)に登録されている「発言」のうち、最新のものから最大20件取得する
        /// </summary>
        public Status[] GetFavorites() {
            return GetFavorites(null, 0);
        }

        /// <summary>
        /// 指定したユーザのfavorites(お気に入り) に登録されている「発言」のうち、最新のものから最大20件取得する
        /// </summary>
        /// <param name="user">「お気に入り」一覧を取得するユーザの ID またはスクリーン名を指定する</param>
        /// <param name="page">(1ページを20件とみなしたときの)ページ番号を指定することで、過去の任意の「お気に入り」一覧を20件単位で取得する</param>
        public Status[] GetFavorites(User user, int page) {
            NameValueCollection options = new NameValueCollection();
            string url = "";
            if(user == null) {
                url = URL_favorites + ".xml";
            } else {
                url = URL_favorites + "/" + user.Id + ".xml";
            }
            if(page != 0) {
                options.Add("page", page.ToString());
            }

            string res = Utility.HttpGet(url, options, this.UserID, this.UserPass);

            return Status.CreateStatuses(res);
        }

        /// <summary>
        /// 指定ステータスを自分の「お気に入り」に登録する。
        /// </summary>
        /// <param name="status">「お気に入り」に登録したいステータス(発言)を指定する</param>
        /// <returns>登録したステータス</returns>
        public Status AddFavorite(Status status) {
            string res = Utility.HttpPost(URL_favorites + "/create/" + status.Id + ".xml", new NameValueCollection(), this.UserID, this.UserPass);
            return new Status(res);
        }

        /// <summary>
        /// 指定ステータスを自分の「お気に入り」から外す。
        /// </summary>
        /// <param name="status">「お気に入り」から外したいステータス(発言)を指定する</param>
        /// <returns>外したステータス</returns>
        public Status RemoveFavorite(Status status) {
            string res = Utility.HttpPost(URL_favorites + "/destroy/" + status.Id + ".xml", new NameValueCollection(), this.UserID, this.UserPass);
            return new Status(res);
        }

        #endregion

        #region Follow関連

        /// <summary>
        /// 指定ユーザを自分の friend (following) にする
        /// </summary>
        /// <returns>followingにしたユーザー。失敗するとnullが返る</returns>
        public User AddFollowing(User user) {
            string res = Utility.HttpPost(URL_friendships_ + "create/" + user.Id + ".xml", new NameValueCollection(), this.UserID, this.UserPass);
            try {
                return new User(res);
            } catch(WebException ex) {
                if(ex.Message.Contains("403")) return null;
                throw ex;
            }
        }

        /// <summary>
        /// 指定ユーザを自分の friend (following) から外す
        /// </summary>
        /// <returns>followingから外したユーザー。失敗するとnullが返る</returns>
        public User RemoveFollowing(User user) {
            string res = Utility.HttpPost(URL_friendships_ + "destroy/" + user.Id + ".xml", new NameValueCollection(), this.UserID, this.UserPass);
            try {
                return new User(res);
            } catch(WebException ex) {
                if(ex.Message.Contains("403")) return null;
                throw ex;
            }
        }

        /// <summary>
        /// 指定した2ユーザの間の friend 関係を調べる
        /// </summary>
        public bool ExistsFollow(User userA, User userB) {
            NameValueCollection options = new NameValueCollection();
            options.Add("user_a", userA.Id.ToString());
            options.Add("user_b", userB.Id.ToString());

            string res = Utility.HttpGet(URL_friendships_ + "exists.xml", options, this.UserID, this.UserPass);

            return res.Contains("true");
        }

        #region GetFollowings()
        /// <summary>
        /// 自分の friend の一覧を(各 friend の最新ステータス付きで)取得する
        /// ただし、この API で取得できるデータは最大100件(100人分)である
        /// </summary>
        public User[] GetFollowings() {
            return GetFollowings(null, 0, false, null);
        }
        /// <summary>
        /// 指定したUserのfriendの一覧を(各 friend の最新ステータス付きで)取得する
        /// ただし、この API で取得できるデータは最大100件(100人分)である
        /// </summary>
        /// <param name="user">friendを取得するUser</param>
        public User[] GetFollowings(User user) {
            return GetFollowings(user, 0, false, null);
        }
        /// <summary>
        /// 自分の friend の一覧を(各 friend の最新ステータス付きで)取得する
        /// ただし、この API で取得できるデータは最大100件(100人分)である
        /// </summary>
        /// <param name="page">(1ページを100件とみなしたときの)ページ番号を指定することで、指定ユーザの friend の一覧を100件単位で取得する</param>
        public User[] GetFollowings(int page) {
            return GetFollowings(null, page, false, null);
        }
        /// <summary>
        /// 指定したUserのfriendの一覧を(各 friend の最新ステータス付きで)取得する
        /// ただし、この API で取得できるデータは最大100件(100人分)である
        /// </summary>
        /// <param name="user">friendを取得するUser</param>
        /// <param name="page">(1ページを100件とみなしたときの)ページ番号を指定することで、指定ユーザの friend の一覧を100件単位で取得する</param>
        public User[] GetFollowings(User user, int page) {
            return GetFollowings(user, page, false, null);
        }
        //todo:since,liteが反映されて無い。
        /// <summary>
        /// 指定したUserのfriendの一覧を(各 friend の最新ステータス付きで)取得する
        /// ただし、この API で取得できるデータは最大100件(100人分)である
        /// </summary>
        /// <param name="user">friendを取得するUser</param>
        /// <param name="page">(1ページを100件とみなしたときの)ページ番号を指定することで、指定ユーザの friend の一覧を100件単位で取得する</param>
        /// <param name="lite">true を指定すると、「各 friend の最新ステータスなし」で friend の一覧を返す</param>
        /// <param name="since">指定した日時以降にステータス が update された friend の一覧を取得する</param>
        public User[] GetFollowings(User user, int page, bool lite, Nullable<DateTime> since) {
#if DEBUG
            NameValueCollection options = new NameValueCollection();
            string url;
            if(user == null) {
                url = URL_statuses_friends + ".xml";
            } else {
                url = URL_statuses_friends + "/" + user.Id + ".xml";
            }
            if(page != 0) {
                options.Add("page", page.ToString());
            }
            if(lite) {
                options.Add("lite", "true");
            }
            if(since != null) {
                options.Add("since", Utility.ConvertDateTime(since.Value));
            }

            string res = Utility.HttpGet(url, options, this.UserID, this.UserPass);

            return User.CreateUsers(res);
#else
            throw new NotImplementedException();
#endif
        }
        #endregion

        #region GetFollowers()
        /// <summary>
        /// 自分の follower の一覧を(各 follower の最新ステータス付きで)取得する
        /// </summary>
        public User[] GetFollowers() {
            return GetFollowers(null, 0, false);
        }
        /// <summary>
        /// 指定したユーザの follower のステータスを取得する (この引数を指定しない場合は、自分の follower のステータスを取得する)
        /// </summary>
        public User[] GetFollowers(User user) {
            return GetFollowers(user, 0, false);
        }
        /// <summary>
        /// 自分の follower の一覧を(各 follower の最新ステータス付きで)取得する
        /// </summary>
        /// <param name="page">(1ページを100件とみなしたときの)ページ番号を指定することで、指定ユーザの follower の一覧を100件単位で取得する</param>
        public User[] GetFollowers(int page) {
            return GetFollowers(null, page, false);
        }
        /// <summary>
        /// 指定したユーザの follower のステータスを取得する (この引数を指定しない場合は、自分の follower のステータスを取得する)
        /// </summary>
        /// <param name="user">指定したユーザの follower のステータスを取得する (この引数を指定しない場合は、自分の follower のステータスを取得する)</param>
        /// <param name="page">(1ページを100件とみなしたときの)ページ番号を指定することで、指定ユーザの follower の一覧を100件単位で取得する</param>
        public User[] GetFollowers(User user, int page) {
            return GetFollowers(user, page, false);
        }
        //todo:liteが反映されない(常にFalse状態)
        /// <summary>
        /// 指定したユーザの follower のステータスを取得する (この引数を指定しない場合は、自分の follower のステータスを取得する)
        /// </summary>
        /// <param name="user">指定したユーザの follower のステータスを取得する (この引数を指定しない場合は、自分の follower のステータスを取得する)</param>
        /// <param name="page">(1ページを100件とみなしたときの)ページ番号を指定することで、指定ユーザの follower の一覧を100件単位で取得する</param>
        /// <param name="lite">true を指定すると、「各 follower の最新ステータスなし」で follower の一覧を返す</param>
        public User[] GetFollowers(User user, int page, bool lite) {
#if DEBUG
            NameValueCollection options = new NameValueCollection();
            string url;
            if(user == null) {
                url = URL_statuses_followers + ".xml";
            } else {
                url = URL_statuses_followers + "/" + user.Id + ".xml";
            }
            if(page != 0) {
                options.Add("page", page.ToString());
            }
            if(lite) {
                options.Add("lite", "true");
            }
            string res = Utility.HttpGet(url, options, this.UserID, this.UserPass);

            return User.CreateUsers(res);
#else
            throw new NotImplementedException();
#endif
        }
        #endregion

        #endregion

        #region タイムライン関連未test
        /// <summary>
        /// 公開ユーザの最新のステータス(発言)を取得する (最大20件)
        /// </summary>
        /// <remarks>
        /// 取得結果はTwitter側で60秒間キャッシュされる。
        /// それよりも短い間隔では取得出来ないので取りこぼしが起き得る
        /// </remarks>
        public Status[] GetPublicTimeline() {
            string url = URL_statuses + "/public_timeline.xml";
            string res = Utility.HttpGet(url, new NameValueCollection(), this.UserID, this.UserPass);

            return Status.CreateStatuses(res);
        }

        #region GetFriendsTimeline()
        /// <summary>
        /// 自分がfollowしているユーザーの過去24時間以内に投稿されたステータスから最大20件を取得する。
        /// </summary>
        public Status[] GetFriendsTimeline() {
            return GetFriendsTimeline(null, null, 0, 0);
        }
        /// <summary>
        /// 自分がfollowしているユーザーの過去24時間以内に投稿されたステータスから最大20件を取得する。
        /// </summary>
        /// <param name="count">
        /// 取得するステータスの件数。最大200。
        /// </param>
        /// <param name="page">
        /// 1から始まるページ数を指定。
        /// </param>
        public Status[] GetFriendsTimeline(int count, int page) {
            return GetFriendsTimeline(null, null, count, page);
        }
        /// <summary>
        /// 自分がfollowしているユーザーの過去24時間以内に投稿されたステータスから最大20件を取得する。
        /// </summary>
        /// <param name="since">指定日時以降のステータスを取得</param>
        public Status[] GetFriendsTimeline(DateTime since) {
            return GetFriendsTimeline(since, null, 0, 0);
        }
        /// <summary>
        /// 自分がfollowしているユーザーの過去24時間以内に投稿されたステータスから最大20件を取得する。
        /// </summary>
        /// <param name="since">指定日時以降のステータスを取得</param>
        /// <param name="count">
        /// 取得するステータスの件数。最大200。
        /// </param>
        /// <param name="page">
        /// 1から始まるページ数を指定。
        /// </param>
        public Status[] GetFriendsTimeline(DateTime since, int count, int page) {
            return GetFriendsTimeline(since, null, count, page);
        }
        /// <summary>
        /// 自分がfollowしているユーザーの過去24時間以内に投稿されたステータスから最大20件を取得する。
        /// </summary>
        ///<param name="since_status">
        /// 指定したステータス以降に投稿されたステータスを取得。
        /// </param>
        public Status[] GetFriendsTimeline(Status since_status) {
            return GetFriendsTimeline(null, since_status, 0, 0);
        }
        /// <summary>
        /// 自分がfollowしているユーザーの過去24時間以内に投稿されたステータスから最大20件を取得する。
        /// </summary>
        /// <param name="since_status">
        /// 指定したステータス以降に投稿されたステータスを取得。
        /// </param>
        /// <param name="count">
        /// 取得するステータスの件数。最大200。
        /// </param>
        /// <param name="page">
        /// 1から始まるページ数を指定。
        /// </param>
        public Status[] GetFriendsTimeline(Status since_status, int count, int page) {
            return GetFriendsTimeline(null, since_status, count, page);
        }
        private Status[] GetFriendsTimeline(Nullable<DateTime> dateTime, Status status, int count, int page) {
            NameValueCollection options = new NameValueCollection();
            if(dateTime != null) {
                options.Add("since", Utility.ConvertDateTime(dateTime.Value));
            }
            if(status != null) {
                options.Add("since_id", status.Id.ToString());
            }
            if(count != 0) {
                options.Add("count", count.ToString());
            }
            if(page != 0) {
                options.Add("page", page.ToString());
            }
            string url = URL_statuses + "/friends_timeline.xml";
            string res = Utility.HttpGet(url, options, this.UserID, this.UserPass);

            return Status.CreateStatuses(res);
        }
        #endregion

        #region GetUserTimeline()
        /// <summary>
        /// 自分のステータスを取得する。
        /// </summary>
        public Status[] GetUserTimeline() {
            return GetUserTimeline(0, null, null, 0);
        }
        /// <summary>
        /// 指定したユーザーの過去24時間以内に update されたステータスを取得する。
        /// </summary>
        /// <param name="user">取得するユーザーのID</param>
        /// <param name="count">取得する件数。0、20より大きい値は無効。</param>
        /// <param name="page">
        /// 1から始まるページ数を指定。
        /// </param>
        public Status[] GetUserTimelint(User user, int page, uint count) {
            return GetUserTimeline(user, count, null, null, page);
        }
        /// <summary>
        /// 自分の過去24時間以内に update されたステータスを取得する。
        /// </summary>
        /// <param name="count">取得する件数。0、20より大きい値は無効。</param>
        /// <param name="page">
        /// 1から始まるページ数を指定。
        /// </param>
        public Status[] GetUserTimeline(int page, uint count) {
            return GetUserTimeline(count, null, null, page);
        }
        /// <summary>
        /// 自分の過去24時間以内に update されたステータスを取得する。
        /// </summary>
        /// <param name="count">取得する件数。0、20より大きい値は無効。</param>
        /// <param name="since">
        /// 指定した日時以降に投稿されたステータスを取得。Status.Created_atプロパティの書式で指定。
        /// </param>
        public Status[] GetUserTimeline(DateTime since, uint count) {
            return GetUserTimeline(count, since, null, 0);
        }
        /// <summary>
        /// 自分の過去24時間以内に update されたステータスを取得する。
        /// </summary>
        /// <param name="count">取得する件数。0、20より大きい値は無効。</param>
        /// <param name="since">
        /// 指定した日時以降に投稿されたステータスを取得。Status.Created_atプロパティの書式で指定。
        /// </param>
        /// <param name="page">
        /// 1から始まるページ数を指定。
        /// </param>
        public Status[] GetUserTimeline(DateTime since, int page, uint count) {
            return GetUserTimeline(count, since, null, page);
        }
        /// <summary>
        ///自分の過去24時間以内に update されたステータスを取得する。
        /// </summary>
        /// <param name="count">取得する件数。0、20より大きい値は無効。</param>
        /// <param name="sinceID">
        /// 指定したステータス以降に投稿されたステータスを取得
        /// </param>
        public Status[] GetUserTimeline(Status sinceID, uint count) {
            return GetUserTimeline(count, null, sinceID, 0);
        }
        /// <summary>
        /// 自分の過去24時間以内に update されたステータスを取得する。
        /// </summary>
        /// <param name="count">取得する件数。0、20より大きい値は無効。</param>
        /// <param name="sinceID">
        /// 指定したステータス以降に投稿されたステータスを取得
        /// </param>
        /// <param name="page">
        /// 1から始まるページ数を指定。
        /// </param>
        public Status[] GetUserTimeline(Status sinceID, int page, uint count) {
            return GetUserTimeline(count, null, sinceID, page);
        }
        /// <summary>
        /// 指定したユーザーの過去24時間以内に update されたステータスを取得する。
        /// </summary>
        /// <param name="user">取得するユーザーのID</param>
        /// <param name="count">取得する件数。0、20より大きい値は無効。</param>
        /// <param name="since">
        /// 指定した日時以降に投稿されたステータスを取得。Status.Created_atプロパティの書式で指定。
        /// </param>
        public Status[] GetUserTimeline(User user, DateTime since, uint count) {
            return GetUserTimeline(user, count, since, null, 0);
        }
        /// <summary>
        /// 指定したユーザーの過去24時間以内に update されたステータスを取得する。
        /// </summary>
        /// <param name="user">取得するユーザーのID</param>
        /// <param name="count">取得する件数。0、20より大きい値は無効。</param>
        /// <param name="since">
        /// 指定した日時以降に投稿されたステータスを取得。Status.Created_atプロパティの書式で指定。
        /// </param>
        /// <param name="page">
        /// 1から始まるページ数を指定。
        /// </param>
        public Status[] GetUserTimeline(User user, DateTime since, int page, uint count) {
            return GetUserTimeline(user, count, since, null, page);
        }
        /// <summary>
        /// 指定したユーザーの過去24時間以内に update されたステータスを取得する。
        /// </summary>
        /// <param name="user">取得するユーザーのID</param>
        /// <param name="count">取得する件数。0、20より大きい値は無効。</param>
        /// <param name="sinceID">
        /// 指定したステータス以降に投稿されたステータスを取得
        /// </param>
        public Status[] GetUserTimeline(User user, Status sinceID, uint count) {
            return GetUserTimeline(user, count, null, sinceID, 0);
        }
        /// <summary>
        /// 指定したユーザーの過去24時間以内に update されたステータスを取得する。
        /// </summary>
        /// <param name="user">取得するユーザーのID</param>
        /// <param name="count">取得する件数。0、20より大きい値は無効。</param>
        /// <param name="sinceID">
        /// 指定したステータス以降に投稿されたステータスを取得
        /// </param>
        /// <param name="page">
        /// 1から始まるページ数を指定。
        /// </param>
        public Status[] GetUserTimeline(User user, Status sinceID, int page, uint count) {
            return GetUserTimeline(user,count, null, sinceID, page);
        }
        /// <summary>
        /// 指定したユーザーの過去24時間以内に update されたステータスから最大20件を取得する。
        /// </summary>
        /// <param name="id">取得するユーザーのID</param>
        public Status[] GetUserTimeline(User id) {
            return GetUserTimeline(id, 0, null, null, 0);
        }
        /// <summary>
        /// 自分の過去24時間以内に update されたステータスを取得する。
        /// </summary>
        /// <param name="count">取得する件数。0以下の値、20より大きい値は無効。</param>
        /// <param name="since">
        /// 指定した日時以降に投稿されたステータスを取得。Status.Created_atプロパティの書式で指定。
        /// </param>
        /// <param name="sinceID">
        /// 指定したステータス以降に投稿されたステータスを取得
        /// </param>
        /// <param name="page">
        /// 1から始まるページ数を指定。
        /// </param>
        private Status[] GetUserTimeline(uint count, Nullable<DateTime> since, Status sinceID, int page) {
            return GetUserTimeline(null, count, since, sinceID, page);
        }
        /// <summary>
        /// 指定したIDのユーザーの過去24時間以内に update されたステータスを取得する。
        /// </summary>
        /// <param name="id">取得するユーザーのID</param>
        /// <param name="count">取得する件数。0、20より大きい値は無効。</param>
        /// <param name="since">
        /// 指定した日時以降に投稿されたステータスを取得。Status.Created_atプロパティの書式で指定。
        /// </param>
        /// <param name="sinceID">
        /// 指定したステータス以降に投稿されたステータスを取得
        /// </param>
        /// <param name="page">
        /// 1から始まるページ数を指定。
        /// </param>
        private Status[] GetUserTimeline(User user, uint count, Nullable<DateTime> since, Status sinceID, int page) {
            string id_ = user == null ? "" : ("/" + user.Id);
            NameValueCollection options = new NameValueCollection();
            options.Add("count", Math.Min(20, Math.Max(1, count)).ToString());
            if(since != null) {
                options.Add("since", Utility.ConvertDateTime(since.Value));
            }
            if(sinceID != null) {
                options.Add("since_id", sinceID.Id.ToString());
            }
            if(page != 0) {
                options.Add("page", page.ToString());
            }
            string res = Utility.HttpGet(URL_statuses + "/user_timeline" + id_ + ".xml", options, this.UserID, this.UserPass);

            return Status.CreateStatuses(res);
        }
        #endregion

        /// <summary>
        /// 指定したIDのステータス(1件)を取得する
        /// </summary>
        /// <param name="statusId">取得するステータスのID</param>
        public Status GetStatus(int statusId) {
            string res = Utility.HttpGet(URL_statuses + "/show/" + statusId.ToString() + ".xml", new NameValueCollection(), this.UserID, this.UserPass);
            XElement xEle = XElement.Parse(res);
            return new Status(xEle);
        }

        /// <summary>
        /// 自分のステータスを投稿する。
        /// </summary>
        /// <param name="text">投稿内容</param>
        /// <returns> 
        /// どうしようか考え中。
        /// とりあえず鯖から貰った内容をそのまま返すようにしてみた。
        /// </returns>
        public Status UpdateStatus(string text) {
            NameValueCollection nvc = new NameValueCollection();
            nvc.Add("status", text);

            string res = Utility.HttpPost(URL_statuses + "/update.xml", nvc, this.UserID, this.UserPass);
            return new Status(res);
        }

        #region GetRepliesStatus()
        /// <summary>
        /// 自分に対する返信(冒頭が @ユーザ名 で始まるステータス)の一覧を取得する (最大20件)
        /// </summary>
        public Status[] GetRepliesStatus() {
            return GetRepliesStatus(null, null, 0);
        }
        /// <summary>
        /// 自分に対する返信(冒頭が @ユーザ名 で始まるステータス)の一覧を取得する (最大20件)
        /// </summary>
        /// <param name="since">
        /// 指定した日時以降に投稿されたステータスを取得。Status.Created_atプロパティの書式で指定。
        /// </param>
        /// <param name="sinceID">
        /// 指定したステータス以降に投稿されたステータスを取得
        /// </param>
        /// <param name="page">
        /// 1から始まるページ数を指定。
        /// </param>
        public Status[] GetRepliesStatus(Nullable<DateTime> since, Status sinceID, int page) {
            NameValueCollection options = new NameValueCollection();
            if(since != null) {
                options.Add("since", Utility.ConvertDateTime(since.Value));
            }
            if(sinceID != null) {
                options.Add("since_id", sinceID.Id.ToString());
            }
            if(page != 0) {
                options.Add("page", page.ToString());
            }
            string res = Utility.HttpGet(URL_statuses + "/replies.xml", options, this.UserID, this.UserPass);

            return Status.CreateStatuses(res);
        }
        #endregion

        //todo:動かない,戻り値不明
        /// <summary>
        /// ステータスを削除する
        /// </summary>
        /// <param name="id">削除するステータスを指定</param>
        /// <returns>
        /// </returns>
        public string DeleteStatus(Status id) {
#if DEBUG
            return Utility.HttpGet(URL_statuses + "/destroy/" + id.Id.ToString() + ".xml", new NameValueCollection(), this.UserID, this.UserPass);
#else
            throw new NotImplementedException();
#endif
        }

        #endregion

        #region ユーザー関連未test

        /// <summary>
        /// 現在、Twitter に login 中のユーザの一覧を(最新ステータス付きで)取得する(最大20人分)
        /// </summary>
        public User[] GetOnlineUsers() {
            string res = Utility.HttpGet(URL_statuses_featured_xml, new NameValueCollection(), this.UserID, this.UserPass);
            return User.CreateUsers(res);
        }

        /// <summary>
        /// 指定ユーザに関する詳細な情報を取得する。
        /// ただし、指定ユーザが自分の friend でない場合、この API の実行は失敗する
        /// </summary>
        /// <param name="userID">ユーザID または id=スクリーン名</param>
        public User GetUserFromUserID(string userID) {
            string url = URL_users + "/show/" + userID + ".xml";
            string res = Utility.HttpGet(url, new NameValueCollection(), this.UserID, this.UserPass);
            XElement xEle = XElement.Parse(res);
            return new User(xEle);
        }
        //todo:test
        /// <summary>
        /// 指定ユーザに関する詳細な情報を取得する。
        /// ただし、指定ユーザが自分の friend でない場合、この API の実行は失敗する
        /// </summary>
        /// <param name="eMail">情報取得対象となるユーザをメールアドレスで指定する。</param>
        public User GetUserFromEMail(string eMail) {
            string url = URL_users + "/show.xml";
            NameValueCollection options = new NameValueCollection();
            options.Add("email", eMail);
            string res = Utility.HttpGet(url, options, this.UserID, this.UserPass);
            XElement xEle = XElement.Parse(res);
            return new User(xEle);
        }

        #endregion
    }
}